function [filterBank obsPointsBinary negPointsBinary...
    obsPoints negPoints threshold...
     Xs Ys Os] = prepareDataForAdaboost(indResponseMatrix,allfilteredPos,allfilteredNeg)
% Changes from original function:
%   - moves filter convolution out of the function

% lattice: subsampled positions, a 2D matrix
% obsPoints: original gabor responses


% each image is represented by a vector of length <dataDimension>
norient = size(allfilteredPos,2);
dataDimension = norient*size(indResponseMatrix,1)*size(indResponseMatrix,2);

disp(['data dimension : ' num2str(dataDimension)]);
nimage = size(allfilteredPos,1);
% <obsPoints> are positive data instances, where each instance contains its Gabor responses 
obsPoints = zeros(nimage,dataDimension);
for indImage = 1:nimage
    tmp = [];
    for indOri = 1:norient % for each orientation
        responseMatrix = allfilteredPos{indImage,indOri}(indResponseMatrix(:,:,3));
        responseVec = reshape(responseMatrix,1,size(responseMatrix,1)*size(responseMatrix,2));
        tmp = [tmp responseVec];
    end
    obsPoints(indImage,:) = tmp(:);
end
clear allfilteredPos

% =====================================================
% Negative images
% =====================================================

nimage = size(allfilteredNeg,1);
negPoints = zeros(nimage,dataDimension);
for indImage = 1:nimage
    tmp = [];
    for indOri = 1:norient
        responseMatrix = allfilteredNeg{indImage,indOri}(indResponseMatrix(:,:,3));
        responseVec = reshape(responseMatrix,1,size(responseMatrix,1)*size(responseMatrix,2));
        tmp = [tmp responseVec];
    end
    negPoints(indImage,:) = tmp(:);
end
clear allfilteredNeg

filterBank = struct('name','gabor','paramRange',[]);
filterBank.paramRange = (1:dataDimension)';

% Train a binary classifier for each Gabor basis response 
threshold = zeros(1,dataDimension);
pol = zeros(1,dataDimension);
for i = 1:dataDimension
    maxResponse = max([obsPoints(:,i);negPoints(:,i)]);
    minResponse = min([obsPoints(:,i);negPoints(:,i)]);
    maxBenefit = 0;
    threshold(i) = -1;
    stepsize = (maxResponse-minResponse)/49;
    for indStep = 0:(50-1)
        indResponse = minResponse + indStep * stepsize;
        % balance data weight
        err = 0.5 * (sum(obsPoints(:,i) <= indResponse))/double(size(obsPoints,1))...
            + 0.5 * (sum(negPoints(:,i)>indResponse))/double(size(negPoints,1));
        benefit = abs(err-0.5);
        if err < 0.5
            tmpPol = 1;
        else
            tmpPol = 0;
        end
        if benefit > maxBenefit
            maxBenefit = benefit;
            threshold(i) = indResponse;
            pol(i) = tmpPol;
        end
    end
end

thresholdMatrix = repmat(threshold,size(obsPoints,1),1);
obsPointsBinary = obsPoints > thresholdMatrix; % thresholded responses (predicted labels) on positive images
thresholdMatrix = repmat(threshold,size(negPoints,1),1);
negPointsBinary = negPoints > thresholdMatrix; % thresholded responses (predicted labels) on positive images


Xs = zeros(1,dataDimension);
Ys = zeros(1,dataDimension);
Os = zeros(1,dataDimension);
offset = 0;
for indOri = 1:norient % for each orientation
    Xs(offset+(1:size(indResponseMatrix,1)*size(indResponseMatrix,2))) = reshape(indResponseMatrix(:,:,1),size(indResponseMatrix,1)*size(indResponseMatrix,2),1)-1;
    Ys(offset+(1:size(indResponseMatrix,1)*size(indResponseMatrix,2))) = reshape(indResponseMatrix(:,:,2),size(indResponseMatrix,1)*size(indResponseMatrix,2),1)-1;
    Os(offset+(1:size(indResponseMatrix,1)*size(indResponseMatrix,2))) = ones(size(indResponseMatrix,1)*size(indResponseMatrix,2),1)*indOri-1;
    offset = offset + size(indResponseMatrix,1)*size(indResponseMatrix,2);
end
